﻿/*After Effects must have keyframes selected from one layer in order to export them as text.  /* This is Beatbar Generator update v0.5 by Quantoz of the Nec and PsychoSplash Beatbar generator v0.4 Alpha */

// Changes:
// - Works directly on Audio Amplitude keys instead of checking key value at each frame – This results in much shorter execution time (in my tests at least)
// - Added auto shift for each duplicated layer, user must define when in layer that is being duplicated (that have beat animation) the beat sound needs to happen
// - Last beat is no longer duplicated (in previous version last beat happens twice at the same time)
// - added minRestTime that prevents creating multiple beat layers on the same beat sound (value of 0.1 prevents creating more than 10 beats per second)
// - added possibility to shift beats one frame before actual sound. This is based on this video: https://youtu.be/2L6nW6uNp9k?t=508, his source research is: www.ncbi.nlm.nih.gov/books/NBK92837 (tldr: vision is processed slower than audio in brain, so vision needs to happen first to be "perfectly" synced with audio)

/*----------------------------Notes----------------------------*/
// Before starting the script always select only the layer that you want to be played at the beat
// If beat detection is not working, go to Audio Amplitude layer and check key values - they must exceed threshold to trigger the script and fall below threshold to be able to detect again
// Audio Amplitude layer must be first on the timeline panel!
// For more accurate beats use extremely high framerates of composition (200 - 250 fps!) - You can place this composition inside your main composition that have standard framerate - it won’t affect performance that much.
// Don’t change framerate during preparation of the beatbar

/*----------------------------settings----------------------------*/ 
var threshold = 2			// Audio level witch that triggers beat detection.
var minRestTime = 0.05		// Time that will be passed after detecting a beat, can help with 2 layers being generated on same beat
var BeatTime = 2.16 // Time in seconds when beat actually happens
var useOneFrameShift = 1	// Do you want to shift beats by one frame?
var TargetFramerate = 60	// Framerate of Your main composition, this is used to better sync animations with beats

/*----------------------------RUN----------------------------*/
 var myComp = app.project.activeItem;	 							// Work in currently active composition
var KeysToSkip = Math.round(minRestTime * myComp.frameRate) ;		// Calculate keys that will be skipped after detecting the beat
var AudioAmpLayer = myComp.layer(1);								// Audio amplitude layer MUST BE FIRST ON THE TIMELINE
var SingleBeatLayer = myComp.selectedLayers[0]; 					// Select layer to be duplicated
var FoundBeats = 0													// Just count how much beats have been generated
var OneFrame = 0													// One frame to shift for better sync

													
if (useOneFrameShift == 1)
{
	OneFrame = 1 / TargetFramerate									
}

// First, few checks before main loop
if (myComp == null || !(myComp instanceof CompItem))
{	alert("A Composition must be active to run this script");}
else
{
	if (AudioAmpLayer == null )	
	{	alert ("Select an Audio Amplitude Layer");}
	else
	{
		if (AudioAmpLayer.effect("Both Channels")== null)
		{	alert ("Audio Amplitude Layer must have Both Channels  effect/slider");}
		else
		{
			var AudioAmpKeys = AudioAmpLayer.effect("Both Channels").property("Slider")
			var AllKeys = AudioAmpKeys.numKeys							// Get the total number of keys to be checked
			if (AllKeys < 1)
			{	alert ("There are no keys to work on!");}
			else
			{
				for (i = 2; i<=AllKeys; i++) 							// Start with 2 because previous key is counted and there is no key no.0
				{
					if ( AudioAmpKeys.keyValue(i)> threshold)			// Check if value is higher than threshold
					{
						if ( AudioAmpKeys.keyValue(i-1)<= threshold)	// And previous must be below threshold
						{
							FoundBeats++
							SingleBeatLayer.startTime = AudioAmpKeys.keyTime(i) - BeatTime - OneFrame	// Get the time of the selected key and move layer to this start point
							SingleBeatLayer.duplicate()													// Duplicate this layer at this time
							i = i + KeysToSkip															// Skip some keys so double detection won’t occur
						}
					}
				}
                   SingleBeatLayer.remove()             // Remove last beat because it is the source beat that is beeing duplicated - its no longer needed
                   FoundBeats--
				alert("Found Beats: " + FoundBeats)
			}
		}
	}
}